/*
 * This Java source file was generated by the Gradle 'init' task.
 */
package hpdos;

import hpdos.grpc.HeartbeatRequest;
import hpdos.grpc.HeartbeatResponse;
import hpdos.grpc.HeartbeatServiceGrpc;
import hpdos.handler.HeartbeatHandler;
import hpdos.handler.NetworkHandler;
import hpdos.lib.MasterFollower;
import hpdos.lib.MemoryStorage;
import hpdos.lib.StorageService;
import hpdos.messageformat.MessageConstants;
import io.grpc.ManagedChannel;
import io.grpc.ManagedChannelBuilder;
import io.grpc.Server;
import io.grpc.ServerBuilder;

import java.io.IOException;
import java.util.HashMap;
import java.util.Timer;
import java.util.TimerTask;
import java.util.UUID;

public class MetadataServer {
    private Server server;
    private final HashMap<String, MasterFollower> followers;
    private final String serverID;
    private boolean isMaster = false;
    private int port;
    private final String host;
    private StorageService storageService;

    public MetadataServer() {
        this.followers = new HashMap<>();
        this.serverID = UUID.randomUUID().toString();
        this.port = 10000 + (int)(Math.random() * 40000);
        this.host = "localhost";
    }

    public String getGreeting() {
        return "Hello World!";
    }

    public boolean startServer() {
        server = ServerBuilder.forPort(ConfigConstants.PORT)
                .addService(new NetworkHandler())
                .addService(new HeartbeatHandler(followers, serverID))
                .build();
        try {
            server.start();
        } catch (IOException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    public void blockForIO() {
        if (server == null)
            return;
        try {
            server.awaitTermination();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    private void announceToMaster() {
        final ManagedChannel channel = ManagedChannelBuilder.
                forAddress(ConfigConstants.HOST, ConfigConstants.PORT)
                .usePlaintext()
                .build();
        HeartbeatServiceGrpc.HeartbeatServiceBlockingStub stub = HeartbeatServiceGrpc.newBlockingStub(channel);
        HeartbeatRequest.Builder heartbeat = HeartbeatRequest.newBuilder();
        heartbeat.setPacketType(MessageConstants.PACKET_METADATA_REQUEST);
        heartbeat.setOperationType(MessageConstants.MASTER_HEARTBEAT);
        heartbeat.setFollowerID(this.serverID);
        heartbeat.setIp(this.host);
        heartbeat.setPort(this.port);
        try {
            HeartbeatResponse response = stub.heartbeat(heartbeat.build());
        } catch (Exception e) {
            System.out.println("Metadata Master not found, electing self as master");
            this.isMaster = true;
            this.port = ConfigConstants.PORT;
        }
        channel.shutdown();

    }

    private void startHeartbeatService() {
        Timer timer = new Timer();
        timer.scheduleAtFixedRate(new TimerTask() {
            @Override
            public void run() {
                announceToMaster();
            }
        }, ConfigConstants.HEARTBEAT_INTERVAL, ConfigConstants.HEARTBEAT_INTERVAL);
    }

    private boolean initStorage(int backend) {
        switch (backend) {
            case ConfigConstants.BACKEND_IN_MEMORY:
                this.storageService = new MemoryStorage();
                return true;
            case ConfigConstants.BINARY_TREE_BACKEND:
            case ConfigConstants.LSM_BACKEND:
            default: return false;
        }
    }

    public static void main(String[] args) {
        MetadataServer metaDataServer = new MetadataServer();

        System.out.println(metaDataServer.getGreeting());
        System.out.println("Starting Metadata service");
        System.out.println("Initialising storage service");
        boolean status = metaDataServer.initStorage(ConfigConstants.BACKEND_IN_MEMORY);
        if (!status) {
            System.out.println("Storage server initialisation error");
            return;
        }
        System.out.println("Searching for MetadataMaster");
        metaDataServer.announceToMaster();
        if (metaDataServer.isMaster) {
            status = metaDataServer.startServer();
            System.out.println("Master ID: " + metaDataServer.serverID);
            if (status) {
                System.out.println("Starting RPC server at: " + ConfigConstants.PORT);
                metaDataServer.blockForIO();
            }
            else
                System.out.println("Failed to create server");
        } else {
            System.out.println("Master Node detected.\nStarting heartbeat service");
            metaDataServer.startHeartbeatService();
        }

    }
}
